//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Device/InstrumentsSet.h
//! @brief     Defines class InstrumentsSet.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_DEVICE_INSTRUMENTSSET_H
#define BORNAGAIN_GUI_MODEL_DEVICE_INSTRUMENTSSET_H

#include "Base/Types/VectorWC.h"
#include <QObject>
#include <QVector>
#include <QXmlStreamWriter>

class InstrumentItem;

class InstrumentsSet : public QObject, public VectorWC<InstrumentItem> {
    Q_OBJECT
public:
    InstrumentsSet();
    ~InstrumentsSet();

    void writeTo(QXmlStreamWriter* w) const;
    void readFrom(QXmlStreamReader* r);

    QVector<InstrumentItem*> instrumentItems() const;

    InstrumentItem* findInstrumentItemById(const QString& instrumentId) const;
    bool instrumentExists(const QString& instrumentId) const;

    QStringList instrumentNames() const;

    //! Set an instrument's name and emit the respective signal.
    void setInstrumentName(InstrumentItem* instrument, const QString& name);

signals:
    void setChanged() const;
    void currentModified() const;
};

#endif // BORNAGAIN_GUI_MODEL_DEVICE_INSTRUMENTSSET_H
