//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Data/DatafilesQModel.cpp
//! @brief     Implements class DatafilesQModel.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Data/DatafilesQModel.h"
#include "GUI/Model/Device/DatafileItem.h"
#include "GUI/Model/Files/DatafilesSet.h"
#include "GUI/Model/Project/ProjectDocument.h"

DatafilesQModel::DatafilesQModel(QObject* parent)
    : QAbstractListModel(parent)
{
}

void DatafilesQModel::injectDatafileItem(DatafileItem* item)
{
    const int row = gDoc->datafiles()->size();
    beginInsertRows({}, row, row);
    gDoc->datafilesModifier()->push_back(item);
    endInsertRows();
}

void DatafilesQModel::removeDatafileItemAt(int row)
{
    ASSERT(0 <= row && row < (int)gDoc->datafiles()->size());
    beginRemoveRows({}, row, row);
    gDoc->datafilesModifier()->delete_at(row);
    endRemoveRows();
}

const DatafileItem* DatafilesQModel::topMostItem() const
{
    if (gDoc->datafiles()->size() > 0)
        return gDoc->datafiles()->at(0);
    return nullptr;
}

int DatafilesQModel::rowCount(const QModelIndex& /*parent*/) const
{
    return gDoc->datafiles()->size();
}

QVariant DatafilesQModel::data(const QModelIndex& index, int role) const
{
    if (!index.isValid() || index.row() >= (int)gDoc->datafiles()->size() || index.row() < 0)
        return {};

    const auto* item = gDoc->datafiles()->at(index.row());
    ASSERT(item);

    if (role == Qt::DisplayRole)
        return item->realItemName();

    if (role == Qt::ToolTipRole)
        return {};

    return {};
}

DatafileItem* DatafilesQModel::itemForIndex(const QModelIndex& index)
{
    if (!index.isValid() || index.row() >= (int)gDoc->datafiles()->size() || index.row() < 0)
        return nullptr;
    return gDoc->datafilesModifier()->at(index.row());
}

QModelIndex DatafilesQModel::indexForItem(const DatafileItem* item) const
{
    for (size_t i = 0; i < gDoc->datafiles()->size(); ++i)
        if (gDoc->datafiles()->at(i) == item)
            return createIndex(i, 0, item);
    return {};
}
