#include "GUI/Model/Device/InstrumentItems.h"
#include "GUI/Model/Device/InstrumentsSet.h"
#include "GUI/Model/Project/ProjectDocument.h"
#include "Tests/GTestWrapper/google_test.h"
#include <QSignalSpy>

Q_DECLARE_METATYPE(const InstrumentItem*)

/* Disabled 18feb24: signal no longer emitted at InstrumentsSet level

//! Checks whether instrumentAddedRemoved will be emitted as expected

TEST(TestInstrumentsSet, instrumentAddedRemoved)
{
    gDoc = std::make_unique<ProjectDocument>();

    QSignalSpy spy(gDoc->instruments(), SIGNAL(setChanged()));
    EXPECT_TRUE(spy.isValid());

    // populating instrument model
    auto* p = new GISASInstrumentItem;
    gDoc->instrumentsModifier()->push_back(p);

    // checking that a signal was emitted about the new instrument
    EXPECT_EQ(spy.count(), 1);

    // removing instrument
    gDoc->instrumentsModifier()->delete_element(p);
    EXPECT_EQ(spy.count(), 2);
}
*/

//! Test whether instrumentChanged will be emitted as expected
TEST(TestInstrumentsSet, instrumentChanged)
{
    qRegisterMetaType<const InstrumentItem*>();
    gDoc = std::make_unique<ProjectDocument>();

    // populating instrument model
    auto* instrument1 = new GISASInstrumentItem;
    gDoc->instrumentsModifier()->push_back(instrument1);
    auto* instrument2 = new SpecularInstrumentItem;
    gDoc->instrumentsModifier()->push_back(instrument2);

    QSignalSpy spy(gDoc->instruments(), SIGNAL(currentModified()));
    EXPECT_TRUE(spy.isValid());

    // change name of instrument 1
    gDoc->instrumentsModifier()->setInstrumentName(instrument1, "A");
    EXPECT_EQ(spy.count(), 1);

    // change other properties, e.g. id
    instrument1->setId("xxxxx");
    emit gDoc->instruments()->currentModified();
    EXPECT_EQ(spy.count(), 2);

    // change name of instrument 2
    gDoc->instrumentsModifier()->setInstrumentName(instrument2, "B");
    EXPECT_EQ(spy.count(), 3);

    // Add another instrument
    auto* instrument3 = new OffspecInstrumentItem;
    gDoc->instrumentsModifier()->push_back(instrument3);

    // Change instrument2
    gDoc->instrumentsModifier()->setInstrumentName(instrument2, "BB");
    EXPECT_EQ(spy.count(), 4);

    // Change instrument3
    gDoc->instrumentsModifier()->setInstrumentName(instrument3, "C");
    EXPECT_EQ(spy.count(), 5);
}
